@@ -1,37 +1,31 @@
+use v5.10;
 use strict;
 use warnings;
 package Tie::Scalar::Callback;
-$Tie::Scalar::Callback::VERSION = '0.06';
+$Tie::Scalar::Callback::VERSION = '0.061';
 use parent 'Tie::Scalar';
+use Carp qw(carp);
 
 #ABSTRACT: a tied scalar which executes a callback everytime it is used
 
 sub TIESCALAR {
-  my ($class, $sub, $val) = @_;
+  my ($class, $sub ) = @_;
 
-  die 'Must provide an anonymous subroutine argument'
-    unless $sub && ref $sub eq 'CODE';
+  die 'Must provide subroutine reference argument'
+    unless $sub && ref $sub eq ref sub {};
 
-  bless {
-    sub => $sub,
-    val => $val,
-  }, $class;
+  bless $sub, $class;
 }
 
 sub STORE {
-  my ($self, $val) = @_;
-  return $self->{sub}($self, 'STORE', $val);
+  carp "You can't assign to this tied scalar";
 }
 
 sub FETCH {
   my ($self) = @_;
-  return $self->{sub}($self, 'FETCH');
+  return $self->();
 }
 
-sub DESTROY {
-  my ($self) = @_;
-  return $self->{sub}($self, 'DESTROY');
-}
 
 1;
 
@@ -47,7 +41,7 @@ Tie::Scalar::Callback - a tied scalar which executes a callback everytime it is
 
 =head1 VERSION
 
-version 0.06
+version 0.061
 
 =head1 SYNOPSIS
 
@@ -55,25 +49,13 @@ version 0.06
 
   # this coderef doubles the scalar's value everytime it's fetched
   my $coderef = sub {
-    my ($self, $event, $val) = @_;
-
-    if ($event eq 'STORE')
-    {
-      $self->{val} = $val;
-    }
-    elsif ($event eq 'FETCH')
-    {
-      my $old_val = $self->{val};
-      $self->{val} *= 2;
-      return $old_val;
-    }
-    else # DESTROY
-    {
-      undef $self;
+  	state $value  = 1/2;
+  	state $factor = 2;
+  	$value *= $factor;
     }
   };
 
-  tie(my $doubler, 'Tie::Scalar::Callback', $coderef, 1);
+  tie(my $doubler, 'Tie::Scalar::Callback', $coderef);
 
   print $doubler; 1
   print $doubler; 2
@@ -82,32 +64,8 @@ version 0.06
 =head1 DESCRIPTION
 
 C<Tie::Scalar::Callback> is a class for creating tied scalars which execute
-a callback everytime an event occurs on the scalar. There are three types of
-event:
-
-=over 4
-
-=item * STORE ($self, 'STORE', $value)
-
-Called anytime a value is assigned to the scalar.
-
-=item * FETCH ($self, 'FETCH')
-
-Called anytime the scalar's value is retrieved.
-
-=item * DESTROY ($self, 'DESTROY')
-
-Called on object destruction.
-
-=back
-
-See the synopsis for an example coderef which handles these events.
-
-=head1 INTERNALS
-
-C<Tie::Scalar::Callback> objects are just anonymous hashes. The coderef is
-stored in C<< $self->{sub} >> and the current value of the scalar in
-C<< $self->{val} >>.
+a callback everytime an event occurs on the scalar. The callback's return
+value becomes the scalar's apparent value.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -133,7 +91,7 @@ L<Tie::Scalar::Ratio|https://metacpan.org/pod/Tie::Scalar::Ratio>
 
 =head1 AUTHOR
 
-David Farrell <dfarrell@cpan.org>
+['David Farrell <dfarrell@cpan.org>', 'brian d foy <bdfoy@cpan.org>']
 
 =head1 COPYRIGHT AND LICENSE
 
